@@ -56,6 +56,13 @@ using namespace log4cxx::spi;
 using namespace log4cxx::config;
 using namespace log4cxx::rolling;
 
+struct DOMConfigurator::DOMConfiguratorPrivate
+{
+	helpers::Properties props;
+	spi::LoggerRepositoryPtr repository;
+	spi::LoggerFactoryPtr loggerFactory;
+};
+
 
 #if APR_HAS_THREADS
 namespace log4cxx
@@ -75,7 +82,7 @@ class XMLWatchdog  : public FileWatchdog
 		*/
 		void doOnChange()
 		{
-			DOMConfigurator().doConfigure(file,
+			DOMConfigurator().doConfigure(file(),
 				LogManager::getLoggerRepository());
 		}
 };
@@ -117,10 +124,12 @@ IMPLEMENT_LOG4CXX_OBJECT(DOMConfigurator)
 #define THREAD_CONFIG_ATTR "threadConfiguration"
 
 DOMConfigurator::DOMConfigurator()
-	: props(), repository()
+	: m_priv(std::make_unique<DOMConfiguratorPrivate>())
 {
 }
 
+DOMConfigurator::~DOMConfigurator() {}
+
 /**
 Used internally to parse appenders by IDREF name.
 */
@@ -353,12 +362,12 @@ void DOMConfigurator::parseErrorHandler(Pool& p,
 			else if (tagName == LOGGER_REF)
 			{
 				LogString loggerName(getAttribute(utf8Decoder, currentElement, REF_ATTR));
-				LoggerPtr logger = repository->getLogger(loggerName, loggerFactory);
+				LoggerPtr logger = m_priv->repository->getLogger(loggerName, m_priv->loggerFactory);
 				eh->setLogger(logger);
 			}
 			else if (tagName == ROOT_REF)
 			{
-				LoggerPtr root = repository->getRootLogger();
+				LoggerPtr root = m_priv->repository->getRootLogger();
 				eh->setLogger(root);
 			}
 		}
@@ -422,7 +431,7 @@ void DOMConfigurator::parseLogger(
 	LogString loggerName = subst(getAttribute(utf8Decoder, loggerElement, NAME_ATTR));
 
 	LogLog::debug(LOG4CXX_STR("Retreiving an instance of Logger."));
-	LoggerPtr logger = repository->getLogger(loggerName, loggerFactory);
+	LoggerPtr logger = m_priv->repository->getLogger(loggerName, m_priv->loggerFactory);
 
 	// Setting up a logger needs to be an atomic operation, in order
 	// to protect potential log operations while logger
@@ -459,8 +468,8 @@ void DOMConfigurator::parseLoggerFactory(
 				className,
 				LoggerFactory::getStaticClass(),
 				0);
-		loggerFactory = log4cxx::cast<LoggerFactory>(obj);
-		PropertySetter propSetter(loggerFactory);
+		m_priv->loggerFactory = log4cxx::cast<LoggerFactory>(obj);
+		PropertySetter propSetter(m_priv->loggerFactory);
 
 		for (apr_xml_elem* currentElement = factoryElement->first_child;
 			currentElement;
@@ -486,7 +495,7 @@ void DOMConfigurator::parseRoot(
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
 {
-	LoggerPtr root = repository->getRootLogger();
+	LoggerPtr root = m_priv->repository->getRootLogger();
 	parseChildrenOfLoggerElement(p, utf8Decoder, rootElement, root, true, doc, appenders);
 }
 
@@ -769,16 +778,16 @@ void DOMConfigurator::setParameter(log4cxx::helpers::Pool& p,
 	propSetter.setProperty(name, value, p);
 }
 
-void DOMConfigurator::doConfigure(const File& filename, spi::LoggerRepositoryPtr repository1)
+spi::ConfigurationStatus DOMConfigurator::doConfigure(const File& filename, spi::LoggerRepositoryPtr repository1)
 {
 	repository1->setConfigured(true);
-	this->repository = repository1;
+	m_priv->repository = repository1;
 	LogString msg(LOG4CXX_STR("DOMConfigurator configuring file "));
 	msg.append(filename.getPath());
 	msg.append(LOG4CXX_STR("..."));
 	LogLog::debug(msg);
 
-	loggerFactory = LoggerFactoryPtr(new DefaultLoggerFactory());
+	m_priv->loggerFactory = std::make_shared<DefaultLoggerFactory>();
 
 	Pool p;
 	apr_file_t* fd;
@@ -787,15 +796,25 @@ void DOMConfigurator::doConfigure(const File& filename, spi::LoggerRepositoryPtr
 
 	if (rv != APR_SUCCESS)
 	{
-		LogString msg2(LOG4CXX_STR("Could not open file ["));
+		// There is not technically an exception thrown here, but this behavior matches
+		// what the PropertyConfigurator does
+		IOException io(rv);
+		LogString msg2(LOG4CXX_STR("Could not read configuration file ["));
 		msg2.append(filename.getPath());
-		msg2.append(LOG4CXX_STR("]."));
+		msg2.append(LOG4CXX_STR("]. "));
+		msg2.append(io.what());
 		LogLog::error(msg2);
+		return spi::ConfigurationStatus::NotConfigured;
 	}
 	else
 	{
 		apr_xml_parser* parser = NULL;
 		apr_xml_doc* doc = NULL;
+
+		LogString debugMsg = LOG4CXX_STR("Loading configuration file [")
+				+ filename.getPath() + LOG4CXX_STR("].");
+		LogLog::debug(debugMsg);
+
 		rv = apr_xml_parse_file(p.getAPRPool(), &parser, &doc, fd, 2000);
 
 		if (rv != APR_SUCCESS)
@@ -817,6 +836,7 @@ void DOMConfigurator::doConfigure(const File& filename, spi::LoggerRepositoryPtr
 			}
 
 			LogLog::error(msg2);
+			return spi::ConfigurationStatus::NotConfigured;
 		}
 		else
 		{
@@ -825,66 +845,68 @@ void DOMConfigurator::doConfigure(const File& filename, spi::LoggerRepositoryPtr
 			parse(p, utf8Decoder, doc->root, doc, appenders);
 		}
 	}
+
+	return spi::ConfigurationStatus::Configured;
 }
 
-void DOMConfigurator::configure(const std::string& filename)
+spi::ConfigurationStatus DOMConfigurator::configure(const std::string& filename)
 {
 	File file(filename);
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 }
 
 #if LOG4CXX_WCHAR_T_API
-void DOMConfigurator::configure(const std::wstring& filename)
+spi::ConfigurationStatus DOMConfigurator::configure(const std::wstring& filename)
 {
 	File file(filename);
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 }
 #endif
 
 #if LOG4CXX_UNICHAR_API
-void DOMConfigurator::configure(const std::basic_string<UniChar>& filename)
+spi::ConfigurationStatus DOMConfigurator::configure(const std::basic_string<UniChar>& filename)
 {
 	File file(filename);
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 }
 #endif
 
 #if LOG4CXX_CFSTRING_API
-void DOMConfigurator::configure(const CFStringRef& filename)
+spi::ConfigurationStatus DOMConfigurator::configure(const CFStringRef& filename)
 {
 	File file(filename);
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 }
 #endif
 
 
-void DOMConfigurator::configureAndWatch(const std::string& filename)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::string& filename)
 {
-	configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
+	return configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
 }
 
 #if LOG4CXX_WCHAR_T_API
-void DOMConfigurator::configureAndWatch(const std::wstring& filename)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::wstring& filename)
 {
-	configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
+	return configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
 }
 #endif
 
 #if LOG4CXX_UNICHAR_API
-void DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filename)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filename)
 {
-	configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
+	return configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
 }
 #endif
 
 #if LOG4CXX_CFSTRING_API
-void DOMConfigurator::configureAndWatch(const CFStringRef& filename)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const CFStringRef& filename)
 {
-	configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
+	return configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
 }
 #endif
 
-void DOMConfigurator::configureAndWatch(const std::string& filename, long delay)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::string& filename, long delay)
 {
 	File file(filename);
 #if APR_HAS_THREADS
@@ -895,17 +917,21 @@ void DOMConfigurator::configureAndWatch(const std::string& filename, long delay)
 		delete xdog;
 	}
 
+	spi::ConfigurationStatus status = DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+
 	xdog = new XMLWatchdog(file);
 	APRInitializer::registerCleanup(xdog);
 	xdog->setDelay(delay);
 	xdog->start();
+
+	return status;
 #else
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 #endif
 }
 
 #if LOG4CXX_WCHAR_T_API
-void DOMConfigurator::configureAndWatch(const std::wstring& filename, long delay)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::wstring& filename, long delay)
 {
 	File file(filename);
 #if APR_HAS_THREADS
@@ -916,18 +942,22 @@ void DOMConfigurator::configureAndWatch(const std::wstring& filename, long delay
 		delete xdog;
 	}
 
+	spi::ConfigurationStatus status = DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+
 	xdog = new XMLWatchdog(file);
 	APRInitializer::registerCleanup(xdog);
 	xdog->setDelay(delay);
 	xdog->start();
+
+	return status;
 #else
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 #endif
 }
 #endif
 
 #if LOG4CXX_UNICHAR_API
-void DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filename, long delay)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filename, long delay)
 {
 	File file(filename);
 #if APR_HAS_THREADS
@@ -938,18 +968,22 @@ void DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filena
 		delete xdog;
 	}
 
+	spi::ConfigurationStatus status = DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+
 	xdog = new XMLWatchdog(file);
 	APRInitializer::registerCleanup(xdog);
 	xdog->setDelay(delay);
 	xdog->start();
+
+	return status;
 #else
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 #endif
 }
 #endif
 
 #if LOG4CXX_CFSTRING_API
-void DOMConfigurator::configureAndWatch(const CFStringRef& filename, long delay)
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const CFStringRef& filename, long delay)
 {
 	File file(filename);
 #if APR_HAS_THREADS
@@ -960,12 +994,16 @@ void DOMConfigurator::configureAndWatch(const CFStringRef& filename, long delay)
 		delete xdog;
 	}
 
+	spi::ConfigurationStatus status = DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+
 	xdog = new XMLWatchdog(file);
 	APRInitializer::registerCleanup(xdog);
 	xdog->setDelay(delay);
 	xdog->start();
+
+	return status;
 #else
-	DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
+	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
 #endif
 }
 #endif
@@ -1026,26 +1064,27 @@ void DOMConfigurator::parse(
 
 	if (!thresholdStr.empty() && thresholdStr != NULL_STRING)
 	{
-		repository->setThreshold(thresholdStr);
-	}
-
-	LogString strstrValue = subst(getAttribute(utf8Decoder, element, STRINGSTREAM_ATTR));
-	LogLog::debug(LOG4CXX_STR("Stringstream =\"") + strstrValue + LOG4CXX_STR("\"."));
-
-	if (!strstrValue.empty() && strstrValue != NULL_STRING)
-	{
-		MessageBufferUseStaticStream();
+		m_priv->repository->setThreshold(thresholdStr);
 	}
 
 	LogString threadSignalValue = subst(getAttribute(utf8Decoder, element, THREAD_CONFIG_ATTR));
-	if( !threadSignalValue.empty() && threadSignalValue != NULL_STRING ){
-		if( threadSignalValue == LOG4CXX_STR("NoConfiguration") ){
+
+	if ( !threadSignalValue.empty() && threadSignalValue != NULL_STRING )
+	{
+		if ( threadSignalValue == LOG4CXX_STR("NoConfiguration") )
+		{
 			helpers::ThreadUtility::configure( ThreadConfigurationType::NoConfiguration );
-		}else if( threadSignalValue == LOG4CXX_STR("BlockSignalsOnly") ){
+		}
+		else if ( threadSignalValue == LOG4CXX_STR("BlockSignalsOnly") )
+		{
 			helpers::ThreadUtility::configure( ThreadConfigurationType::BlockSignalsOnly );
-		}else if( threadSignalValue == LOG4CXX_STR("NameThreadOnly") ){
+		}
+		else if ( threadSignalValue == LOG4CXX_STR("NameThreadOnly") )
+		{
 			helpers::ThreadUtility::configure( ThreadConfigurationType::NameThreadOnly );
-		}else if( threadSignalValue == LOG4CXX_STR("BlockSignalsAndNameThread") ){
+		}
+		else if ( threadSignalValue == LOG4CXX_STR("BlockSignalsAndNameThread") )
+		{
 			helpers::ThreadUtility::configure( ThreadConfigurationType::BlockSignalsAndNameThread );
 		}
 	}
@@ -1085,7 +1124,7 @@ LogString DOMConfigurator::subst(const LogString& value)
 {
 	try
 	{
-		return OptionConverter::substVars(value, props);
+		return OptionConverter::substVars(value, m_priv->props);
 	}
 	catch (IllegalArgumentException& e)
 	{
