@@ -20,23 +20,43 @@
 #include <apr_strings.h>
 #include <log4cxx/helpers/exception.h>
 #include <log4cxx/helpers/transcoder.h>
+#include <log4cxx/private/action_priv.h>
+#include <log4cxx/helpers/loglog.h>
 
 using namespace log4cxx;
 using namespace log4cxx::rolling;
 using namespace log4cxx::helpers;
 
+#define priv static_cast<GZCompressActionPrivate*>(m_priv.get())
+
+struct GZCompressAction::GZCompressActionPrivate : public ActionPrivate
+{
+	GZCompressActionPrivate( const File& toRename,
+		const File& renameTo,
+		bool deleteSource):
+		source(toRename), destination(renameTo), deleteSource(deleteSource) {}
+
+	const File source;
+	File destination;
+	bool deleteSource;
+	bool throwIOExceptionOnForkFailure = true;
+};
+
 IMPLEMENT_LOG4CXX_OBJECT(GZCompressAction)
 
 GZCompressAction::GZCompressAction(const File& src,
 	const File& dest,
 	bool del)
-	: source(src), destination(dest), deleteSource(del)
+	: Action(std::make_unique<GZCompressActionPrivate>(
+			  src, dest, del))
 {
 }
 
+GZCompressAction::~GZCompressAction() {}
+
 bool GZCompressAction::execute(log4cxx::helpers::Pool& p) const
 {
-	if (source.exists(p))
+	if (priv->source.exists(p))
 	{
 		apr_pool_t* aprpool = p.getAPRPool();
 		apr_procattr_t* attr;
@@ -67,7 +87,7 @@ bool GZCompressAction::execute(log4cxx::helpers::Pool& p) const
 		apr_file_t* child_out;
 		apr_int32_t flags = APR_FOPEN_READ | APR_FOPEN_WRITE |
 			APR_FOPEN_CREATE | APR_FOPEN_TRUNCATE;
-		stat = destination.open(&child_out, flags, APR_OS_DEFAULT, p);
+		stat = priv->destination.open(&child_out, flags, APR_OS_DEFAULT, p);
 
 		if (stat != APR_SUCCESS)
 		{
@@ -97,21 +117,39 @@ bool GZCompressAction::execute(log4cxx::helpers::Pool& p) const
 			}
 		}
 
+		priv->destination.setAutoDelete(true);
+
 		const char** args = (const char**)
 			apr_palloc(aprpool, 4 * sizeof(*args));
 		int i = 0;
 		args[i++] = "gzip";
 		args[i++] = "-c";
-		args[i++] = Transcoder::encode(source.getPath(), p);
+		args[i++] = Transcoder::encode(priv->source.getPath(), p);
 		args[i++] = NULL;
 
-
 		apr_proc_t pid;
 		stat = apr_proc_create(&pid, "gzip", args, NULL, attr, aprpool);
 
-		if (stat != APR_SUCCESS)
+		if (stat != APR_SUCCESS && priv->throwIOExceptionOnForkFailure)
 		{
 			throw IOException(stat);
+		}else if(stat != APR_SUCCESS && !priv->throwIOExceptionOnForkFailure)
+		{
+			/* If we fail here (to create the gzip child process),
+			 * skip the compression and consider the rotation to be
+			 * otherwise successful. The caller has already rotated
+			 * the log file (`source` here refers to the
+			 * uncompressed, rotated path, and `destination` the
+			 * same path with `.gz` appended). Remove the empty
+			 * destination file and leave source as-is.
+			 */
+			LogLog::warn(LOG4CXX_STR("Failed to fork gzip during log rotation; leaving log file uncompressed"));
+			stat = apr_file_close(child_out);
+			if (stat != APR_SUCCESS)
+			{
+				LogLog::warn(LOG4CXX_STR("Failed to close abandoned .gz file; ignoring"));
+			}
+			return true;
 		}
 
 		apr_proc_wait(&pid, NULL, NULL, APR_WAIT);
@@ -122,9 +160,11 @@ bool GZCompressAction::execute(log4cxx::helpers::Pool& p) const
 			throw IOException(stat);
 		}
 
-		if (deleteSource)
+		priv->destination.setAutoDelete(false);
+
+		if (priv->deleteSource)
 		{
-			source.deleteFile(p);
+			priv->source.deleteFile(p);
 		}
 
 		return true;
@@ -133,3 +173,7 @@ bool GZCompressAction::execute(log4cxx::helpers::Pool& p) const
 	return false;
 }
 
+void GZCompressAction::setThrowIOExceptionOnForkFailure(bool throwIO){
+	priv->throwIOExceptionOnForkFailure = throwIO;
+}
+
