@@ -14,10 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#if defined(_MSC_VER)
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
 
 #include <log4cxx/logstring.h>
 #include <log4cxx/pattern/lineseparatorpatternconverter.h>
@@ -40,7 +36,7 @@ LineSeparatorPatternConverter::LineSeparatorPatternConverter() :
 PatternConverterPtr LineSeparatorPatternConverter::newInstance(
 	const std::vector<LogString>& /* options */)
 {
-	static PatternConverterPtr instance(new LineSeparatorPatternConverter());
+	static PatternConverterPtr instance = std::make_shared<LineSeparatorPatternConverter>();
 	return instance;
 }
 
