@@ -40,6 +40,7 @@
 
 #define LOG4CXX 1
 #include <log4cxx/helpers/aprinitializer.h>
+#include <apr.h>
 
 
 using namespace log4cxx;
@@ -66,7 +67,7 @@ class PropertyWatchdog  : public FileWatchdog
 		*/
 		void doOnChange()
 		{
-			PropertyConfigurator().doConfigure(file,
+			PropertyConfigurator().doConfigure(file(),
 				LogManager::getLoggerRepository());
 		}
 };
@@ -88,7 +89,7 @@ PropertyConfigurator::~PropertyConfigurator()
 	delete registry;
 }
 
-void PropertyConfigurator::doConfigure(const File& configFileName,
+spi::ConfigurationStatus PropertyConfigurator::doConfigure(const File& configFileName,
 	spi::LoggerRepositoryPtr hierarchy)
 {
 	hierarchy->setConfigured(true);
@@ -100,43 +101,48 @@ void PropertyConfigurator::doConfigure(const File& configFileName,
 		InputStreamPtr inputStream = InputStreamPtr( new FileInputStream(configFileName) );
 		props.load(inputStream);
 	}
-	catch (const IOException&)
+	catch (const IOException& ex)
 	{
 		LogLog::error(((LogString) LOG4CXX_STR("Could not read configuration file ["))
-			+ configFileName.getPath() + LOG4CXX_STR("]."));
-		return;
+			+ configFileName.getPath() + LOG4CXX_STR("].") + ": " + ex.what());
+		return spi::ConfigurationStatus::NotConfigured;
 	}
 
 	try
 	{
-		doConfigure(props, hierarchy);
+		LogString debugMsg = LOG4CXX_STR("Loading configuration file [")
+				+ configFileName.getPath() + LOG4CXX_STR("].");
+		LogLog::debug(debugMsg);
+		return doConfigure(props, hierarchy);
 	}
 	catch (const std::exception& ex)
 	{
 		LogLog::error(((LogString) LOG4CXX_STR("Could not parse configuration file ["))
 			+ configFileName.getPath() + LOG4CXX_STR("]."), ex);
 	}
+
+	return spi::ConfigurationStatus::NotConfigured;
 }
 
-void PropertyConfigurator::configure(const File& configFilename)
+spi::ConfigurationStatus PropertyConfigurator::configure(const File& configFilename)
 {
-	PropertyConfigurator().doConfigure(configFilename, LogManager::getLoggerRepository());
+	return PropertyConfigurator().doConfigure(configFilename, LogManager::getLoggerRepository());
 }
 
-void PropertyConfigurator::configure(helpers::Properties& properties)
+spi::ConfigurationStatus PropertyConfigurator::configure(helpers::Properties& properties)
 {
-	PropertyConfigurator().doConfigure(properties, LogManager::getLoggerRepository());
+	return PropertyConfigurator().doConfigure(properties, LogManager::getLoggerRepository());
 }
 
 #if APR_HAS_THREADS
-void PropertyConfigurator::configureAndWatch(const File& configFilename)
+spi::ConfigurationStatus PropertyConfigurator::configureAndWatch(const File& configFilename)
 {
-	configureAndWatch(configFilename, FileWatchdog::DEFAULT_DELAY);
+	return configureAndWatch(configFilename, FileWatchdog::DEFAULT_DELAY);
 }
 
 
 
-void PropertyConfigurator::configureAndWatch(
+spi::ConfigurationStatus PropertyConfigurator::configureAndWatch(
 	const File& configFilename, long delay)
 {
 	if (pdog)
@@ -145,14 +151,18 @@ void PropertyConfigurator::configureAndWatch(
 		delete pdog;
 	}
 
+	spi::ConfigurationStatus stat = PropertyConfigurator().doConfigure(configFilename, LogManager::getLoggerRepository());
+
 	pdog = new PropertyWatchdog(configFilename);
 	APRInitializer::registerCleanup(pdog);
 	pdog->setDelay(delay);
 	pdog->start();
+
+	return stat;
 }
 #endif
 
-void PropertyConfigurator::doConfigure(helpers::Properties& properties,
+spi::ConfigurationStatus PropertyConfigurator::doConfigure(helpers::Properties& properties,
 	spi::LoggerRepositoryPtr hierarchy)
 {
 	hierarchy->setConfigured(true);
@@ -177,22 +187,22 @@ void PropertyConfigurator::doConfigure(helpers::Properties& properties,
 			+ LOG4CXX_STR("]."));
 	}
 
-	static const LogString STRINGSTREAM_KEY(LOG4CXX_STR("log4j.stringstream"));
-	LogString strstrValue(properties.getProperty(STRINGSTREAM_KEY));
-
-	if (strstrValue == LOG4CXX_STR("static"))
-	{
-		MessageBufferUseStaticStream();
-	}
-
 	LogString threadConfigurationValue(properties.getProperty(LOG4CXX_STR("log4j.threadConfiguration")));
-	if( threadConfigurationValue == LOG4CXX_STR("NoConfiguration") ){
+
+	if ( threadConfigurationValue == LOG4CXX_STR("NoConfiguration") )
+	{
 		helpers::ThreadUtility::configure( ThreadConfigurationType::NoConfiguration );
-	}else if( threadConfigurationValue == LOG4CXX_STR("BlockSignalsOnly") ){
+	}
+	else if ( threadConfigurationValue == LOG4CXX_STR("BlockSignalsOnly") )
+	{
 		helpers::ThreadUtility::configure( ThreadConfigurationType::BlockSignalsOnly );
-	}else if( threadConfigurationValue == LOG4CXX_STR("NameThreadOnly") ){
+	}
+	else if ( threadConfigurationValue == LOG4CXX_STR("NameThreadOnly") )
+	{
 		helpers::ThreadUtility::configure( ThreadConfigurationType::NameThreadOnly );
-	}else if( threadConfigurationValue == LOG4CXX_STR("BlockSignalsAndNameThread") ){
+	}
+	else if ( threadConfigurationValue == LOG4CXX_STR("BlockSignalsAndNameThread") )
+	{
 		helpers::ThreadUtility::configure( ThreadConfigurationType::BlockSignalsAndNameThread );
 	}
 
@@ -205,6 +215,8 @@ void PropertyConfigurator::doConfigure(helpers::Properties& properties,
 	// We don't want to hold references to appenders preventing their
 	// destruction.
 	registry->clear();
+
+	return spi::ConfigurationStatus::Configured;
 }
 
 void PropertyConfigurator::configureLoggerFactory(helpers::Properties& props)
@@ -429,15 +441,23 @@ AppenderPtr PropertyConfigurator::parseAppender(
 	// Appender was not previously initialized.
 	LogString prefix = APPENDER_PREFIX + appenderName;
 	LogString layoutPrefix = prefix + LOG4CXX_STR(".layout");
-	LogString rollingPolicyKey = prefix + LOG4CXX_STR(".rollingPolicy");
-	LogString triggeringPolicyKey = prefix + LOG4CXX_STR(".triggeringPolicy");
 
 	std::shared_ptr<Object> obj =
 		OptionConverter::instantiateByKey(
 			props, prefix, Appender::getStaticClass(), 0);
 	appender = log4cxx::cast<Appender>( obj );
 
-	if (appender == 0)
+	// Map obsolete DailyRollingFileAppender property configuration
+	if (!appender &&
+		StringHelper::endsWith(OptionConverter::findAndSubst(prefix, props), LOG4CXX_STR("DailyRollingFileAppender")))
+	{
+		appender = std::make_shared<RollingFileAppender>();
+		auto datePattern = OptionConverter::findAndSubst(prefix + LOG4CXX_STR(".datePattern"), props);
+		if (!datePattern.empty())
+			props.put(prefix + LOG4CXX_STR(".fileDatePattern"), datePattern);
+	}
+
+	if (!appender)
 	{
 		LogLog::error((LogString) LOG4CXX_STR("Could not instantiate appender named \"")
 			+ appenderName + LOG4CXX_STR("\"."));
@@ -464,7 +484,6 @@ AppenderPtr PropertyConfigurator::parseAppender(
 				LogLog::debug((LogString) LOG4CXX_STR("Parsing layout options for \"")
 					+ appenderName + LOG4CXX_STR("\"."));
 
-				//configureOptionHandler(layout, layoutPrefix + ".", props);
 				PropertySetter::setProperties(layout, props, layoutPrefix + LOG4CXX_STR("."), p);
 				LogLog::debug((LogString) LOG4CXX_STR("End of parsing for \"")
 					+ appenderName +  LOG4CXX_STR("\"."));
@@ -472,38 +491,45 @@ AppenderPtr PropertyConfigurator::parseAppender(
 		}
 
 		RollingFileAppenderPtr rolling = log4cxx::cast<rolling::RollingFileAppender>(appender);
-		if(rolling)
+		if (rolling)
 		{
-			RollingPolicyPtr rollingPolicy;
-			std::shared_ptr<Object> rolling_obj =
-				OptionConverter::instantiateByKey(
-					props, rollingPolicyKey, RollingPolicy::getStaticClass(), 0);
-			rollingPolicy = log4cxx::cast<RollingPolicy>( rolling_obj );
-			if(rollingPolicy)
+			LogString rollingPolicyKey = prefix + LOG4CXX_STR(".rollingPolicy");
+			if (!OptionConverter::findAndSubst(rollingPolicyKey, props).empty())
 			{
-				rolling->setRollingPolicy(rollingPolicy);
+				RollingPolicyPtr rollingPolicy;
+				std::shared_ptr<Object> rolling_obj =
+					OptionConverter::instantiateByKey(
+						props, rollingPolicyKey, RollingPolicy::getStaticClass(), 0);
+				rollingPolicy = log4cxx::cast<RollingPolicy>( rolling_obj );
+				if(rollingPolicy)
+				{
+					rolling->setRollingPolicy(rollingPolicy);
 
-				LogLog::debug((LogString) LOG4CXX_STR("Parsing rolling policy options for \"")
-					+ appenderName + LOG4CXX_STR("\"."));
-				PropertySetter::setProperties(rollingPolicy, props, rollingPolicyKey + LOG4CXX_STR("."), p);
+					LogLog::debug((LogString) LOG4CXX_STR("Parsing rolling policy options for \"")
+						+ appenderName + LOG4CXX_STR("\"."));
+					PropertySetter::setProperties(rollingPolicy, props, rollingPolicyKey + LOG4CXX_STR("."), p);
+				}
 			}
 
-			TriggeringPolicyPtr triggeringPolicy;
-			std::shared_ptr<Object> triggering_obj =
-				OptionConverter::instantiateByKey(
-					props, triggeringPolicyKey, TriggeringPolicy::getStaticClass(), 0);
-			triggeringPolicy = log4cxx::cast<TriggeringPolicy>( triggering_obj );
-			if(triggeringPolicy)
+			LogString triggeringPolicyKey = prefix + LOG4CXX_STR(".triggeringPolicy");
+			if (!OptionConverter::findAndSubst(triggeringPolicyKey, props).empty())
 			{
-				rolling->setTriggeringPolicy(triggeringPolicy);
+				TriggeringPolicyPtr triggeringPolicy;
+				std::shared_ptr<Object> triggering_obj =
+					OptionConverter::instantiateByKey(
+						props, triggeringPolicyKey, TriggeringPolicy::getStaticClass(), 0);
+				triggeringPolicy = log4cxx::cast<TriggeringPolicy>( triggering_obj );
+				if(triggeringPolicy)
+				{
+					rolling->setTriggeringPolicy(triggeringPolicy);
 
-				LogLog::debug((LogString) LOG4CXX_STR("Parsing triggering policy options for \"")
-					+ appenderName + LOG4CXX_STR("\"."));
-				PropertySetter::setProperties(triggeringPolicy, props, triggeringPolicyKey + LOG4CXX_STR("."), p);
+					LogLog::debug((LogString) LOG4CXX_STR("Parsing triggering policy options for \"")
+						+ appenderName + LOG4CXX_STR("\"."));
+					PropertySetter::setProperties(triggeringPolicy, props, triggeringPolicyKey + LOG4CXX_STR("."), p);
+				}
 			}
 		}
 
-		//configureOptionHandler((OptionHandler) appender, prefix + _T("."), props);
 		PropertySetter::setProperties(appender, props, prefix + LOG4CXX_STR("."), p);
 		LogLog::debug((LogString) LOG4CXX_STR("Parsed \"")
 			+ appenderName + LOG4CXX_STR("\" options."));
