@@ -14,9 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#if defined(_MSC_VER)
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
 
 #define __STDC_CONSTANT_MACROS
 #include <log4cxx/net/socketappenderskeleton.h>
@@ -27,41 +24,34 @@
 #include <log4cxx/helpers/transcoder.h>
 #include <log4cxx/helpers/bytearrayoutputstream.h>
 #include <log4cxx/helpers/threadutility.h>
+#include <log4cxx/private/appenderskeleton_priv.h>
+#include <log4cxx/private/socketappenderskeleton_priv.h>
 #include <functional>
+#include <chrono>
 
 using namespace log4cxx;
 using namespace log4cxx::helpers;
 using namespace log4cxx::net;
 
-SocketAppenderSkeleton::SocketAppenderSkeleton(int defaultPort, int reconnectionDelay1)
-	:  remoteHost(),
-	   address(),
-	   port(defaultPort),
-	   reconnectionDelay(reconnectionDelay1),
-	   locationInfo(false),
-	   thread()
+#define _priv static_cast<SocketAppenderSkeletonPriv*>(m_priv.get())
+
+SocketAppenderSkeleton::SocketAppenderSkeleton(int defaultPort, int reconnectionDelay)
+    : AppenderSkeleton(std::make_unique<SocketAppenderSkeletonPriv>(defaultPort, reconnectionDelay))
 {
 }
 
-SocketAppenderSkeleton::SocketAppenderSkeleton(InetAddressPtr address1, int port1, int delay)
-	:
-	remoteHost(),
-	address(address1),
-	port(port1),
-	reconnectionDelay(delay),
-	locationInfo(false),
-	thread()
+SocketAppenderSkeleton::SocketAppenderSkeleton(helpers::InetAddressPtr address, int port, int reconnectionDelay)
+    : AppenderSkeleton(std::make_unique<SocketAppenderSkeletonPriv>(address, port, reconnectionDelay))
 {
-	remoteHost = this->address->getHostName();
 }
 
-SocketAppenderSkeleton::SocketAppenderSkeleton(const LogString& host, int port1, int delay)
-	:   remoteHost(host),
-		address(InetAddress::getByName(host)),
-		port(port1),
-		reconnectionDelay(delay),
-		locationInfo(false),
-		thread()
+SocketAppenderSkeleton::SocketAppenderSkeleton(const LogString& host, int port, int reconnectionDelay)
+    : AppenderSkeleton(std::make_unique<SocketAppenderSkeletonPriv>(host, port, reconnectionDelay))
+{
+}
+
+SocketAppenderSkeleton::SocketAppenderSkeleton(std::unique_ptr<SocketAppenderSkeletonPriv> priv)
+	:  AppenderSkeleton (std::move(priv))
 {
 }
 
@@ -78,33 +68,33 @@ void SocketAppenderSkeleton::activateOptions(Pool& p)
 
 void SocketAppenderSkeleton::close()
 {
-	std::lock_guard<std::recursive_mutex> lock(mutex);
+	std::lock_guard<std::recursive_mutex> lock(_priv->mutex);
 
-	if (closed)
+	if (_priv->closed)
 	{
 		return;
 	}
 
-	closed = true;
-	cleanUp(pool);
+	_priv->closed = true;
+	cleanUp(_priv->pool);
 
 	{
-		std::unique_lock<std::mutex> lock2(interrupt_mutex);
-		interrupt.notify_all();
+		std::unique_lock<std::mutex> lock2(_priv->interrupt_mutex);
+		_priv->interrupt.notify_all();
 	}
 
-	if ( thread.joinable() )
+	if ( _priv->thread.joinable() )
 	{
-		thread.join();
+		_priv->thread.join();
 	}
 }
 
 void SocketAppenderSkeleton::connect(Pool& p)
 {
-	if (address == 0)
+	if (_priv->address == 0)
 	{
 		LogLog::error(LogString(LOG4CXX_STR("No remote host is set for Appender named \"")) +
-			name + LOG4CXX_STR("\"."));
+			_priv->name + LOG4CXX_STR("\"."));
 	}
 	else
 	{
@@ -112,15 +102,15 @@ void SocketAppenderSkeleton::connect(Pool& p)
 
 		try
 		{
-			SocketPtr socket(new Socket(address, port));
+			SocketPtr socket = Socket::create(_priv->address, _priv->port);
 			setSocket(socket, p);
 		}
 		catch (SocketException& e)
 		{
 			LogString msg = LOG4CXX_STR("Could not connect to remote log4cxx server at [")
-				+ address->getHostName() + LOG4CXX_STR("].");
+				+ _priv->address->getHostName() + LOG4CXX_STR("].");
 
-			if (reconnectionDelay > 0)
+			if (_priv->reconnectionDelay > 0)
 			{
 				msg += LOG4CXX_STR(" We will try again later. ");
 			}
@@ -157,52 +147,40 @@ void SocketAppenderSkeleton::setOption(const LogString& option, const LogString&
 
 void SocketAppenderSkeleton::fireConnector()
 {
-	std::lock_guard<std::recursive_mutex> lock(mutex);
+	std::lock_guard<std::recursive_mutex> lock(_priv->mutex);
 
-	if( thread.joinable() ){
-		// This should only get called if the thread has already exited.
-		// We could have a potential bug if fireConnector is called while
-		// the thread is waiting for a connection
-		thread.join();
-	}
-
-	if ( !thread.joinable() )
+	if ( !_priv->thread.joinable() )
 	{
 		LogLog::debug(LOG4CXX_STR("Connector thread not alive: starting monitor."));
 
-		thread = ThreadUtility::instance()->createThread( LOG4CXX_STR("SocketAppend"), &SocketAppenderSkeleton::monitor, this );
+		_priv->thread = ThreadUtility::instance()->createThread( LOG4CXX_STR("SocketAppend"), &SocketAppenderSkeleton::monitor, this );
 	}
 }
 
 void SocketAppenderSkeleton::monitor()
 {
 	SocketPtr socket;
-	bool isClosed = closed;
+	bool isClosed = _priv->closed;
 
 	while (!isClosed)
 	{
 		try
 		{
-			if (!closed)
+			if (!_priv->closed)
 			{
 				LogLog::debug(LogString(LOG4CXX_STR("Attempting connection to "))
-					+ address->getHostName());
-				socket = SocketPtr(new Socket(address, port));
+					+ _priv->address->getHostName());
+				socket = Socket::create(_priv->address, _priv->port);
 				Pool p;
 				setSocket(socket, p);
 				LogLog::debug(LOG4CXX_STR("Connection established. Exiting connector thread."));
 				return;
 			}
 		}
-		catch (InterruptedException&)
-		{
-			LogLog::debug(LOG4CXX_STR("Connector interrupted.  Leaving loop."));
-			return;
-		}
 		catch (ConnectException&)
 		{
 			LogLog::debug(LOG4CXX_STR("Remote host ")
-				+ address->getHostName()
+				+ _priv->address->getHostName()
 				+ LOG4CXX_STR(" refused connection."));
 		}
 		catch (IOException& e)
@@ -211,16 +189,16 @@ void SocketAppenderSkeleton::monitor()
 			log4cxx::helpers::Transcoder::decode(e.what(), exmsg);
 
 			LogLog::debug(((LogString) LOG4CXX_STR("Could not connect to "))
-				+ address->getHostName()
+				+ _priv->address->getHostName()
 				+ LOG4CXX_STR(". Exception is ")
 				+ exmsg);
 		}
 
-		std::unique_lock<std::mutex> lock( interrupt_mutex );
-		interrupt.wait_for( lock, std::chrono::milliseconds( reconnectionDelay ),
+		std::unique_lock<std::mutex> lock( _priv->interrupt_mutex );
+		_priv->interrupt.wait_for( lock, std::chrono::milliseconds( _priv->reconnectionDelay ),
 			std::bind(&SocketAppenderSkeleton::is_closed, this) );
 
-		isClosed = closed;
+		isClosed = _priv->closed;
 	}
 
 	LogLog::debug(LOG4CXX_STR("Exiting Connector.run() method."));
@@ -228,5 +206,46 @@ void SocketAppenderSkeleton::monitor()
 
 bool SocketAppenderSkeleton::is_closed()
 {
-	return closed;
+	return _priv->closed;
+}
+
+void SocketAppenderSkeleton::setRemoteHost(const LogString& host)
+{
+	_priv->address = helpers::InetAddress::getByName(host);
+	_priv->remoteHost.assign(host);
+}
+
+const LogString& SocketAppenderSkeleton::getRemoteHost() const
+{
+	return _priv->remoteHost;
+}
+
+void SocketAppenderSkeleton::setPort(int port1)
+{
+	_priv->port = port1;
+}
+
+int SocketAppenderSkeleton::getPort() const
+{
+	return _priv->port;
+}
+
+void SocketAppenderSkeleton::setLocationInfo(bool locationInfo1)
+{
+	_priv->locationInfo = locationInfo1;
+}
+
+bool SocketAppenderSkeleton::getLocationInfo() const
+{
+	return _priv->locationInfo;
+}
+
+void SocketAppenderSkeleton::setReconnectionDelay(int reconnectionDelay1)
+{
+	_priv->reconnectionDelay = reconnectionDelay1;
+}
+
+int SocketAppenderSkeleton::getReconnectionDelay() const
+{
+	return _priv->reconnectionDelay;
 }
