@@ -18,12 +18,6 @@
 #ifndef _LOG4CXX_ASYNC_APPENDER_H
 #define _LOG4CXX_ASYNC_APPENDER_H
 
-#if defined(_MSC_VER)
-	#pragma warning ( push )
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
-
 #include <log4cxx/appenderskeleton.h>
 #include <log4cxx/helpers/appenderattachableimpl.h>
 #include <deque>
@@ -57,6 +51,9 @@ class LOG4CXX_EXPORT AsyncAppender :
 	public virtual spi::AppenderAttachable,
 	public virtual AppenderSkeleton
 {
+	protected:
+		struct AsyncAppenderPriv;
+
 	public:
 		DECLARE_LOG4CXX_OBJECT(AsyncAppender)
 		BEGIN_LOG4CXX_CAST_MAP()
@@ -80,25 +77,25 @@ class LOG4CXX_EXPORT AsyncAppender :
 		 *
 		 * @param newAppender appender to add, may not be null.
 		*/
-		void addAppender(const AppenderPtr newAppender);
+		void addAppender(const AppenderPtr newAppender) override;
 
-		virtual void doAppend(const spi::LoggingEventPtr& event,
-			log4cxx::helpers::Pool& pool1);
+		void doAppend(const spi::LoggingEventPtr& event,
+			helpers::Pool& pool1) override;
 
-		void append(const spi::LoggingEventPtr& event, log4cxx::helpers::Pool& p);
+		void append(const spi::LoggingEventPtr& event, helpers::Pool& p) override;
 
 		/**
 		Close this <code>AsyncAppender</code> by interrupting the
 		dispatcher thread which will process all pending events before
 		exiting.
 		*/
-		void close();
+		void close() override;
 
 		/**
 		 * Get iterator over attached appenders.
 		 * @return list of all attached appenders.
 		*/
-		AppenderList getAllAppenders() const;
+		AppenderList getAllAppenders() const override;
 
 		/**
 		 * Get appender by name.
@@ -106,7 +103,7 @@ class LOG4CXX_EXPORT AsyncAppender :
 		 * @param name name, may not be null.
 		 * @return matching appender or null.
 		*/
-		AppenderPtr getAppender(const LogString& name) const;
+		AppenderPtr getAppender(const LogString& name) const override;
 
 		/**
 		 * Gets whether the location of the logging request call
@@ -120,25 +117,25 @@ class LOG4CXX_EXPORT AsyncAppender :
 		* @param appender appender.
 		* @return true if attached.
 		*/
-		bool isAttached(const AppenderPtr appender) const;
+		bool isAttached(const AppenderPtr appender) const override;
 
-		virtual bool requiresLayout() const;
+		bool requiresLayout() const override;
 
 		/**
 		 * Removes and closes all attached appenders.
 		*/
-		void removeAllAppenders();
+		void removeAllAppenders() override;
 
 		/**
 		 * Removes an appender.
 		 * @param appender appender to remove.
 		*/
-		void removeAppender(const AppenderPtr appender);
+		void removeAppender(const AppenderPtr appender) override;
 		/**
 		* Remove appender by name.
 		* @param name name.
 		*/
-		void removeAppender(const LogString& name);
+		void removeAppender(const LogString& name) override;
 
 		/**
 		* The <b>LocationInfo</b> attribute is provided for compatibility
@@ -183,113 +180,12 @@ class LOG4CXX_EXPORT AsyncAppender :
 		 * @param option property name.
 		 * @param value property value.
 		 */
-		void setOption(const LogString& option, const LogString& value);
+		void setOption(const LogString& option, const LogString& value) override;
 
 
 	private:
 		AsyncAppender(const AsyncAppender&);
 		AsyncAppender& operator=(const AsyncAppender&);
-		/**
-		 * The default buffer size is set to 128 events.
-		*/
-		enum { DEFAULT_BUFFER_SIZE = 128 };
-
-		/**
-		 * Event buffer.
-		*/
-#if defined(NON_BLOCKING)
-		boost::lockfree::queue<log4cxx::spi::LoggingEvent* > buffer;
-		std::atomic<size_t> discardedCount;
-#else
-		LoggingEventList buffer;
-#endif
-
-		/**
-		 *  Mutex used to guard access to buffer and discardMap.
-		 */
-		std::mutex bufferMutex;
-
-#if defined(NON_BLOCKING)
-		::log4cxx::helpers::Semaphore bufferNotFull;
-		::log4cxx::helpers::Semaphore bufferNotEmpty;
-#else
-		std::condition_variable bufferNotFull;
-		std::condition_variable bufferNotEmpty;
-#endif
-		class DiscardSummary
-		{
-			private:
-				/**
-				 * First event of the highest severity.
-				*/
-				::log4cxx::spi::LoggingEventPtr maxEvent;
-
-				/**
-				* Total count of messages discarded.
-				*/
-				int count;
-
-			public:
-				/**
-				 * Create new instance.
-				 *
-				 * @param event event, may not be null.
-				*/
-				DiscardSummary(const ::log4cxx::spi::LoggingEventPtr& event);
-				/** Copy constructor.  */
-				DiscardSummary(const DiscardSummary& src);
-				/** Assignment operator. */
-				DiscardSummary& operator=(const DiscardSummary& src);
-
-				/**
-				 * Add discarded event to summary.
-				 *
-				 * @param event event, may not be null.
-				*/
-				void add(const ::log4cxx::spi::LoggingEventPtr& event);
-
-				/**
-				 * Create event with summary information.
-				 *
-				 * @return new event.
-				 */
-				::log4cxx::spi::LoggingEventPtr createEvent(::log4cxx::helpers::Pool& p);
-
-				static
-				::log4cxx::spi::LoggingEventPtr createEvent(::log4cxx::helpers::Pool& p,
-					size_t discardedCount);
-		};
-
-		/**
-		  * Map of DiscardSummary objects keyed by logger name.
-		*/
-		typedef std::map<LogString, DiscardSummary> DiscardMap;
-		DiscardMap* discardMap;
-
-		/**
-		 * Buffer size.
-		*/
-		int bufferSize;
-
-		/**
-		 * Nested appenders.
-		*/
-		helpers::AppenderAttachableImplPtr appenders;
-
-		/**
-		 *  Dispatcher.
-		 */
-		std::thread dispatcher;
-
-		/**
-		 * Should location info be included in dispatched messages.
-		*/
-		bool locationInfo;
-
-		/**
-		 * Does appender block when buffer is full.
-		*/
-		bool blocking;
 
 		/**
 		 *  Dispatch routine.
@@ -300,10 +196,5 @@ class LOG4CXX_EXPORT AsyncAppender :
 LOG4CXX_PTR_DEF(AsyncAppender);
 }  //  namespace log4cxx
 
-#if defined(_MSC_VER)
-	#pragma warning ( pop )
-#endif
-
-
 #endif//  _LOG4CXX_ASYNC_APPENDER_H
 
