@@ -18,12 +18,6 @@
 #if !defined(_LOG4CXX_LOGUNIT_H)
 #define _LOG4CXX_LOGUNIT_H
 
-
-#if defined(_MSC_VER)
-	#pragma warning (push)
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
 #include "abts.h"
 #include <exception>
 #include <map>
@@ -274,10 +268,5 @@ class RegisterDisabledSuite
 #define LOGUNIT_ASSERT_EQUAL_SRCL(  expected, actual, srcLine)  assertEquals(expected, actual, #expected, #actual, srcLine)
 #define LOGUNIT_FAIL(msg)                                       throw LogUnit::AssertException(msg, __LINE__)
 
-
-#if defined(_MSC_VER)
-	#pragma warning (pop)
-#endif
-
 #endif
 
