@@ -47,7 +47,9 @@ public:
 
 	void tearDown()
 	{
-		if (auto rep = logger->getLoggerRepository())
+		auto rep = logger->getLoggerRepository();
+
+		if (rep)
 		{
 			rep->resetConfiguration();
 		}
