@@ -15,10 +15,6 @@
  * limitations under the License.
  */
 
-#include <log4cxx/net/socketappender.h>
-#include <log4cxx/patternlayout.h>
-#include <log4cxx/basicconfigurator.h>
-#include <log4cxx/helpers/serversocket.h>
 #include "../appenderskeletontestcase.h"
 #include "apr.h"
 
